package com.qmth.themis.business.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.themis.business.bean.exam.ExamPrepareBean;
import com.qmth.themis.business.cache.RedisKeyHelper;
import com.qmth.themis.business.cache.bean.ExamActivityCacheBean;
import com.qmth.themis.business.cache.bean.ExamCourseCacheBean;
import com.qmth.themis.business.cache.bean.ExamPaperCacheBean;
import com.qmth.themis.business.cache.bean.ExamStudentCacheBean;
import com.qmth.themis.business.config.SystemConfig;
import com.qmth.themis.business.dao.TEExamMapper;
import com.qmth.themis.business.entity.TEExam;
import com.qmth.themis.business.service.*;
import com.qmth.themis.business.util.OssUtil;
import com.qmth.themis.business.util.RedisUtil;
import com.qmth.themis.common.exception.BusinessException;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * @Description: 考试批次 服务实现类
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2020/6/25
 */
@Service
public class TEExamServiceImpl extends ServiceImpl<TEExamMapper, TEExam> implements TEExamService {

    @Resource
    TEExamMapper teExamMapper;

    @Resource
    TEExamActivityService teExamActivityService;

    @Resource
    TEExamStudentService teExamStudentService;

    @Resource
    TEExamCourseService teExamCourseService;

    @Resource
    TEExamPaperService teExamPaperService;

    @Resource
    TOeExamRecordService toeExamRecordService;

    @Resource
    RedisUtil redisUtil;

    @Resource
    SystemConfig systemConfig;

    /**
     * 查询考试批次
     *
     * @param iPage
     * @param id
     * @param code
     * @param name
     * @param mode
     * @param enable
     * @return
     */
    @Override
    public IPage<TEExam> examQuery(IPage<Map> iPage, Long id, String code, String name, Integer mode, Integer enable) {
        return teExamMapper.examQuery(iPage, id, code, name, mode, enable);
    }

    /**
     * 获取考试待考列表
     *
     * @param studentId
     * @param examId
     * @param orgId
     * @return
     */
    @Override
    public List<Map> getWaitingExam(Long studentId, Long examId, Long orgId) {
        List<Map> list = teExamMapper.getWaitingExam(studentId, examId, orgId);
        if (Objects.nonNull(list) && list.size() > 0) {
            for (int i = 0; i < list.size(); i++) {
                Map m = list.get(i);
                List<Map> teExamActivityList = teExamActivityService.getWaitingExam(studentId,
                        Long.parseLong(String.valueOf(m.get("id"))),
                        Long.parseLong(String.valueOf(m.get("examActivityId"))));
                m.put("activities", teExamActivityList);
            }
        }
        return list;
    }

    @Transactional
    @Override
    public ExamPrepareBean prepare(Long studentId, Long examStudentId) {
        ExamPrepareBean ret = null;
        ExamStudentCacheBean es = null;
        es = (ExamStudentCacheBean) redisUtil.get(RedisKeyHelper.examStudentCacheKey(examStudentId));
        if (es == null) {
            es = teExamStudentService.getExamStudnetCacheBean(examStudentId);
        }
        if (es == null) {
            throw new BusinessException("未找到考试");
        }

        if (studentId.equals(es.getStudentId())) {
            throw new BusinessException("考生Id和当前登录用户不一致");
        }
        if (es.getLeftExamCount() == 0) {
            throw new BusinessException("没有剩余考试次数");
        }
        Long activityId = es.getExamActivityId();
        ExamActivityCacheBean ac = teExamActivityService.getExamActivityCacheBean(activityId);
        if (ac == null) {
            throw new BusinessException("未找到场次");
        }
        Date now = new Date();
        Long start = ac.getStartTime().getTime() - (ac.getPrepareSeconds() * 1000);
        Long end = ac.getStartTime().getTime() + (ac.getOpeningSeconds() * 1000);
        if (now.getTime() < start) {
            throw new BusinessException("没有到允许开考的时间");
        }
        if (now.getTime() > end) {
            throw new BusinessException("允许开考的时间已结束");
        }
        ExamCourseCacheBean ec = teExamCourseService.getExamCourseCacheBean(es.getExamId(), es.getCourseCode());
        if (ec == null) {
            throw new BusinessException("未找到考试科目");
        }
        if (ec.getPaperIds() == null) {
            throw new BusinessException("考试科目未绑定试卷");
        }
        if (ec.getHasAnswer() == null || ec.getHasAnswer().intValue() == 0) {
            throw new BusinessException("考试科目答案未补全");
        }
        // 根据权重选中试卷
        Long paperId = ec.getPaperIds().get(getPaperByWeight(ec.getPaperWeight()));
        ExamPaperCacheBean ep = teExamPaperService.getExamPaperCacheBean(paperId);
        if (ep == null) {
            throw new BusinessException("未找到试卷：" + paperId);
        }
        if (StringUtils.isBlank(ep.getAnswerPath())) {
            throw new BusinessException("试卷答案未上传：" + paperId);
        }

        // 写入次数
        Integer serialNumber = es.getCurrentSerialNumber();
        if (serialNumber == null) {
            serialNumber = 0;
        }
        es.setCurrentSerialNumber(serialNumber + 1);

        Long recordId = toeExamRecordService.saveByPrepare(es.getExamId(), es.getExamActivityId(), examStudentId,
                paperId, es.getCurrentSerialNumber());

        Integer leftExamCount = es.getLeftExamCount();
        es.setLeftExamCount(leftExamCount - 1);

        es.setCurrentRecordId(recordId);

        ExamPrepareBean prepare = new ExamPrepareBean();
        prepare.setRecordId(recordId);
        prepare.setAudioPlayTime(ep.getAudioPlayTime());
        prepare.setHasAudio((ep.getHasAudio() == null || ep.getHasAudio().intValue() == 0 ? false : true));
        prepare.setObjectiveShuffle(
                (ec.getObjectiveShuffle() == null || ec.getObjectiveShuffle().intValue() == 0 ? false : true));
        prepare.setOptionShuffle(
                (ec.getOptionShuffle() == null || ec.getOptionShuffle().intValue() == 0 ? false : true));
        String paperurl = OssUtil.getUrlForPrivateBucket(systemConfig.getOssEnv(3), ep.getPaperPath());
        String structurl = OssUtil.getUrlForPrivateBucket(systemConfig.getOssEnv(3), ep.getStructPath());
        prepare.setPaperUrl(paperurl);
        prepare.setStructUrl(structurl);

        // 更新考生缓存
        redisUtil.set(RedisKeyHelper.examStudentCacheKey(examStudentId), es);

        return ret;
    }

    /**
     * 根据设定几率取出一套试卷
     */
    private int getPaperByWeight(List<Double> paperWeight) {
        // 从1开始
        double r = Math.random();
        for (int i = 0; i < paperWeight.size(); i++) {
            r -= paperWeight.get(i);
            if (r <= 0.0d) {
                return i;// 选中
            }
        }

        return -1;
    }
}
