package com.qmth.themis.task.quartz;

import cn.hutool.core.date.DateUtil;
import com.qmth.themis.business.constant.SystemConstant;
import com.qmth.themis.business.threadPool.MyThreadPool;
import com.qmth.themis.business.util.RedisUtil;
import com.qmth.themis.mq.dto.MqDto;
import com.qmth.themis.mq.service.ProducerServer;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.scheduling.quartz.QuartzJobBean;

import javax.annotation.Resource;
import java.util.Date;
import java.util.Map;
import java.util.Objects;

/**
 * @Description: mq job
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2020/7/1
 */
public class MqJob extends QuartzJobBean {
    private final static Logger log = LoggerFactory.getLogger(MqJob.class);

    @Resource
    RedisUtil redisUtil;

    @Resource
    ProducerServer producerServer;

    @Resource
    MyThreadPool myThreadPool;

    @Override
    protected void executeInternal(JobExecutionContext context) throws JobExecutionException {
        myThreadPool.arbitratePoolTaskExecutor.execute(() -> {
            log.info("session_topic_job进来了");
            this.assembleJob(SystemConstant.SESSION_TOPIC_BUFFER_LIST);
            log.info("userLog_topic_job进来了");
            this.assembleJob(SystemConstant.USERLOG_TOPIC_BUFFER_LIST);
            log.info("studentLog_topic_job进来了");
            this.assembleJob(SystemConstant.STUDENTLOG_TOPIC_BUFFER_LIST);
            log.info("task_topic_job进来了");
            this.assembleJob(SystemConstant.TASKLOG_TOPIC_BUFFER_LIST);
        });
        myThreadPool.arbitratePoolTaskExecutor.execute(() -> {
            log.info("websocket_un_normal_topic_job进来了");
            this.assembleDelayJob(SystemConstant.WEBSOCKET_UN_NORMAL_LOG_TOPIC_BUFFER_LIST);
        });
    }

    /**
     * 组装job
     *
     * @param redisKey
     */
    public void assembleJob(String redisKey) {
        Long size = redisUtil.getHashSize(redisKey);
        if (Objects.nonNull(size) && size.longValue() > 0) {
            log.info("redisKey:{}缓冲区的消息数为:{}", redisKey, size);
            Map map = redisUtil.getHashEntries(redisKey);
            map.forEach((k, v) -> {
                MqDto mqDto = (MqDto) v;
                producerServer.sendOneWay(mqDto);
            });
        }
    }

    /**
     * 组装延时job
     *
     * @param redisKey
     */
    public void assembleDelayJob(String redisKey) {
        Long size = redisUtil.getHashSize(redisKey);
        if (Objects.nonNull(size) && size.longValue() > 0) {
            log.info("redisKey:{}缓冲区的消息数为:{}", redisKey, size);
            Map map = redisUtil.getHashEntries(redisKey);
            map.forEach((k, v) -> {
                MqDto mqDto = (MqDto) v;
                Map<String, Object> prop = mqDto.getProperties();
                Long mqExecTime = Long.parseLong(String.valueOf(prop.get("mqExecTime")));
                //mq为延时消息，所以每分钟扫描时有可能mq消息还未执行，所以加上mq消息等级和当前时间对比，如超过mq的延时时间一分钟则执行
                if (Objects.nonNull(mqExecTime) && (mqExecTime - System.currentTimeMillis()) / 1000 / 60 <= -1) {
                    producerServer.asyncDelayMsg(mqDto);
                }
            });
        }
    }
}
